% Main routine for log spectral subtraction. 
% 
% Call this routine with the first argument set to a vector containing
% a time-domain audio signal.  The vector should be a column vector
% (i.e., a matrix with one column).  This routine returns a vector
% containing the processed time-domain output.
%
% I have only tested this code with values
% of winLength such that the number of points
% in the FFT (equal to Fs * winLength) is a power of 2.
% from David Gelbart ICSI

function r = main_mod(input, Fs, winLength)

% code modified to use only a single mean computation over the entire
% utterance. 

if (nargin==1)
  Fs = 8000; % Fs=sampling frequency in Hz
% else
%   % just echo the Fs passed in
%   Fs
end


if (nargin<=2)
  winLength = 1.024;     % length of analysis window in seconds
	  	          % chose 1.024 to make N=power of 2
% else
%   winLength;
end



% Make sure there are an integer number of window steps in the sample
% by padding at the end if necessary.  
N=Fs*winLength;
step=N/4;
lengthOfTailPad = step - rem( length(input) - N, step );
tailPad = input( length(input) - lengthOfTailPad + 1 : length(input) );
input = [input ; tailPad ];

% Analysis.
%'analysis'
spectra = analyze(input,N);

% Separation into phase and magnitude.
%'separation'

eps = 1e-9;
logmagspectra = log(abs(spectra)+eps); % Epsilon value to avoid log of zero error
phasespectra = angle(spectra);

% Normalization.
%'normalization'

% To use the full signal for mean computation instead of the sliding window
% logmagspectraN=normalize(logmagspectra, leftWing, rightWing);
logmagspectraN=normalize_mod(logmagspectra);

% Combination of phase and magnitude.
%'combination'
spectraN = exp(logmagspectraN) .* exp(i*phasespectra);

% Resynthesis.
%'resynthesis'
r = real ( resynth(spectraN) );  

% Throw away any tail padding we may have added at the beginning
r = r(1 : length(r) - lengthOfTailPad);

% Return a column vector.
r = transpose(r);
