% This is a version of normalize.m which does minimum phase
% reconstruction to assign a phase spectrum to the mean log
% magnitude spectrum.
%
% Input should be a log magnitude spectrum 
function spectraN = phase_normalize(spectra, leftWing, rightWing)

N = length( spectra(1, :)  ); % number of samples in each DFT
Nspectra = length( spectra(:, 1)  ); % number of spectra 

if (Nspectra < leftWing + 1 + rightWing)
   error('Not enough input data to estimate mean.');
end

for i=leftWing+1:Nspectra-rightWing
  window = spectra(i-leftWing:i+rightWing, :); 
  s = mean(window);    

  % s is the real part of the complex log cepstrum.
  % We now reconstruct the imaginary part (corresponding to the 
  % phase spectrum). The imaginary part 
  % will correspond to a minimum phase reverberation filter h(n).

  % This code is based on steps 1-3 on page 672 in Chapter 10 of the 
  % book "Discrete-time signal processing" by Alan V. Oppenheim
  % and Ronald W. Schafer.   I believe I used the 1989
  % edition of the book.  I think later editions
  % don't have the full details on this that are in the 1989 edition.

  xe = real(ifft(s));  % xe should be real, any complex part is noise
  x(1) = xe(1);  
  x(N/2+1) = xe(N/2+1);
  x(2:N/2) = 2*xe(2:N/2);
  x(N/2+2:N) = 0;
  s = fft(x);

  specmean(i,:) = s;

end

% for beginning and end of signal, use first available mean
for i=1:leftWing
  spectraN(i,:) = spectra(i,:) - specmean(leftWing+1,:);
end
for i=Nspectra-rightWing+1:Nspectra
  spectraN(i,:) = spectra(i,:) - specmean(Nspectra-rightWing,:);
end

% otherwise use mean that was calculated centered on the frame we 
% are normalizing
for i=leftWing+1:Nspectra-rightWing   
  spectraN(i,:) = spectra(i,:) - specmean(i,:);
end


