% Perform overlap-add resynthesis, assuming a window step of
% N/4.  With this window step, there will be 4 overlapping
% Hanning windows at each sample (except at the beginning
% and end).  Summing Hanning windows overlapped like that will sum up
% to a flat line which is why this approach works.

function r = resynth(spectra)

Npad = 3; % see comments in analyze.m
N = length( spectra(1, :)  ); % number of samples in each DFT
Nspectra = length( spectra(:, 1)  ); % number of spectra 

% array of resynthesized samples
r = zeros(1,(N/4)*(Nspectra+3)); % assumes overlap factor = 1/4

first = 1; % start index of this block of resynthesized samples
for i=1:Nspectra 
   last = first + N-1; % end index of this block of resynthesized samples

   s = ifft(spectra(i,:).').';

   r(first:last) = r(first:last) + 0.5*s;

   first = first + N/4; % assumes overlap factor = 1/4
end

% Throw away the padding that we assume analyze.m has added.	
r = r(Npad*N/4 + 1 : length(r) - Npad*N/4);

